/*
Copyright (C) 2003 - Derek John Evans 

This file is part of Yeti3D Portable Engine

Yeti3D is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Original Source: 2003 - Derek J. Evans <derek@theteahouse.com.au>
Prepared for public release: 10/24/2003 - Derek J. Evans <derek@theteahouse.com.au>
*/

/*
**  This demo using SDL was made by OneThirty8, but is basically a
**  modification of the DirectDraw demo by Derek J. Evans.  The idea here
**  is to do basically the same thing, but also to be portable to
**  Mac OSX and Linux.  Surprisingly, this runs fairly well on the
**  Dreamcast without using the 3D hardware.
*/

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <SDL/SDL.h>
#include "game.h"
#include "flush_uppermem_cache.h"   // Squidgehack

framebuffer_t framebuffer; //gives us somewhere to draw our stuff.
yeti_t yeti;
int oldTick = 0;		// Used to help limit the framerate.
int done = 0;			// Keeps track of whether or not we're still playing.
SDL_Surface *screen;	// our video surface

SDL_Event event;

void Yeti_Cleanup();    // Lineus - added cleanup function
//void Yeti_Blit_HUD(SDL_Surface* display, char* hud_filename);  // Lineus - blit a HUD on the display

void sdl3d_flip() // checks whether we're ready to draw a frame, and if so it draws it.
{
	int currentTick;
	int waitTicks;
    
	currentTick=SDL_GetTicks();

	oldTick += YETI_VIEWPORT_INTERVAL;
	waitTicks = (oldTick + (YETI_VIEWPORT_INTERVAL)) - currentTick;
	oldTick = currentTick;

	/* Only loop through the game loop and draw if enough
	   time has passed since the last frame.
	   The viewport_to_video function takes the frame we've drawm to
	   our 15-bpp 'framebuffer' in memory, converts it to 16-bit,
	   and throws it onto our actual video surface.  Look at yeti.c
	   to see what is going on under the hood here. */
	//if(waitTicks>0)
	//{
	//	SDL_Delay(waitTicks);
	//}

	game_loop(&yeti);
    
	viewport_to_video( (rgb555_t*)screen->pixels, screen->pitch, &yeti.viewport, 0xf800, 0x07e0, 0x001f);
    flush_uppermem_cache(&screen->pixels, &screen->pixels + 320*240, 0);     // Squidgehack
	SDL_Flip(screen);
}



int main(int argc, char *argv[])
{
	system("/sbin/rmmod mmuhack");         // Squidgehack
    system("/sbin/insmod mmuhack.o");
    
    Uint8* keys;  // to keep track of keypresses later on.

	//Initialize SDL video.
	if ( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER|SDL_INIT_JOYSTICK) < 0 )
	{
		printf("Unable to init SDL: %s\n", SDL_GetError());
		exit(1);
	}
	atexit(Yeti_Cleanup);
	
	// Squidgehack
    int mmufd = open("/dev/mmuhack", O_RDWR);
    if(mmufd < 0)
    {
        printf("MMU hack failed");
    }
    else
    {
        printf("MMU hack loaded");
        close(mmufd);
    }
	// Squidgehack --^
	
	SDL_Joystick* joystick = SDL_JoystickOpen(0);
	
	// set up our video surface to have the same width and height as our viewport
	// Lineus - changed to SDL_HWSURFACE because of Squidgehack
	screen = SDL_SetVideoMode(YETI_VIEWPORT_WIDTH, YETI_VIEWPORT_HEIGHT, 16, SDL_HWSURFACE);
	if ( screen == NULL )
	{
		printf("Unable to set %d, %d 16-bit video: %s\n",YETI_VIEWPORT_WIDTH,
               YETI_VIEWPORT_HEIGHT, SDL_GetError());
		exit(1);
	}
	SDL_ShowCursor(0);
	SDL_JoystickEventState(SDL_ENABLE);
  
	//Initiallize Yeti3D
	yeti_init(&yeti, &framebuffer, &framebuffer, textures, palette, lua);
	game_init(&yeti, &e1m1);

	while(done==0) // While we're not done playing, keep looping.
	{
		sdl3d_flip();
		while ( SDL_PollEvent(&event) ) // Here, if the user presses START, we quit.
		{
			if ( event.type == SDL_QUIT )  {  done = 1;  }
			if ( event.type == SDL_JOYBUTTONDOWN )
			{
				if ( event.jbutton.button == GP2X_BUTTON_START ) { done = 1; }
			}
		}  
		// update the keyboard structure - which pipes the events
		yeti.keyboard.up     = (SDL_JoystickGetButton(joystick,GP2X_BUTTON_UP)
                              ||SDL_JoystickGetButton(joystick,GP2X_BUTTON_UPLEFT)
                              ||SDL_JoystickGetButton(joystick,GP2X_BUTTON_UPRIGHT));
		yeti.keyboard.down   = (SDL_JoystickGetButton(joystick,GP2X_BUTTON_DOWN)
                              ||SDL_JoystickGetButton(joystick,GP2X_BUTTON_DOWNLEFT)
                              ||SDL_JoystickGetButton(joystick,GP2X_BUTTON_DOWNRIGHT));
		yeti.keyboard.left   = (SDL_JoystickGetButton(joystick,GP2X_BUTTON_LEFT)
                              ||SDL_JoystickGetButton(joystick,GP2X_BUTTON_UPLEFT)
                              ||SDL_JoystickGetButton(joystick,GP2X_BUTTON_DOWNLEFT));
		yeti.keyboard.right  = (SDL_JoystickGetButton(joystick,GP2X_BUTTON_RIGHT)
                              ||SDL_JoystickGetButton(joystick,GP2X_BUTTON_UPRIGHT)
                              ||SDL_JoystickGetButton(joystick,GP2X_BUTTON_DOWNRIGHT));
        yeti.keyboard.click  = SDL_JoystickGetButton(joystick,GP2X_BUTTON_CLICK);   //toggle crawl
        yeti.keyboard.l      = SDL_JoystickGetButton(joystick,GP2X_BUTTON_L);
        yeti.keyboard.r      = SDL_JoystickGetButton(joystick,GP2X_BUTTON_R);
		yeti.keyboard.a      = SDL_JoystickGetButton(joystick,GP2X_BUTTON_A);       //fire
		yeti.keyboard.b      = SDL_JoystickGetButton(joystick,GP2X_BUTTON_B);       //jump
		yeti.keyboard.y      = SDL_JoystickGetButton(joystick,GP2X_BUTTON_Y);       //look up
		yeti.keyboard.x      = SDL_JoystickGetButton(joystick,GP2X_BUTTON_X);       //look down
		yeti.keyboard.volup  = SDL_JoystickGetButton(joystick,GP2X_BUTTON_VOLUP);
		yeti.keyboard.voldown= SDL_JoystickGetButton(joystick,GP2X_BUTTON_VOLDOWN);
		yeti.keyboard.select = SDL_JoystickGetButton(joystick,GP2X_BUTTON_SELECT);
		yeti.keyboard.start  = SDL_JoystickGetButton(joystick,GP2X_BUTTON_START);    //exit
	}

	return 0;
}

// Lineus - handy way to clean up... called at exit of program
// place all cleanup code here - SDL_Quit(), etc... 
void Yeti_Cleanup()
{
    SDL_Quit();
    system("/sbin/rmmod mmuhack");  // unload Squidgehack
    system("exec /usr/gp2x/gp2xmenu");  // start menu
}

// Lineus - Blit a HUD on the screen -- works but crashes and REEEEEAAAAALLLLLY slow
/*
void Yeti_Blit_HUD(SDL_Surface* display,char* hud_filename)
{
    SDL_Surface *hud;       // HUD Surface
    SDL_Rect src,dest;      // source and destination rectangles
    hud = SDL_LoadBMP(hud_filename);    // BMP file HUD to load
    SDL_SetColorKey(hud,SDL_SRCCOLORKEY,0);     //make black transparent
    src.w=dest.w=hud->w;    // set widths
    src.h=dest.h=hud->h;    // set heights
    src.x=src.y=dest.x=dest.y=0;
    SDL_BlitSurface(hud, &src, display, &dest);  // blit
    //SDL_UpdateRect(display,0,0,0,0);
}
*/
